/*
    Copyright 2009 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "blowfish.h"

#ifdef WIN32
#else
    #define stricmp strcasecmp
    #define stristr strcasestr
#endif

typedef unsigned char   u8;
typedef unsigned int    u32;



#define VER         "0.1"



void xfire_pwd(u8 *pwd);
void std_err(void);
void wait_exit(int ret);



int main(int argc, char *argv[]) {
    FILE    *fd;
    u8      buff[1024],
            *fname,
            *p;

    fputs("\n"
        "Xfire password decrypter "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <XfireUser.ini/password>\n"
            "\n", argv[0]);
        wait_exit(1);
    }
    fname = argv[1];

    fd = fopen(fname, "rb");
    if(!fd) {
        p = strdup(fname);  // not needed
        xfire_pwd(p);
        printf("\n  PASSWORD: %s\n", p);
    } else {
        while(fgets(buff, sizeof(buff), fd)) {
            for(p = buff; *p && (*p != '\r') && (*p != '\n'); p++);
            *p = 0;

            p = strchr(buff, '=');
            if(!p) continue;
            *p++ = 0;
            if(stristr(buff, "encrypted") || stristr(buff, "EPW")) {
                xfire_pwd(p);
                printf("  %-20s %s\n", buff, p);
            }
            // Xfire stores only the custom hash (not spent time on it due to the lack of interest) of the password and then when the client logins it sends a salted hash
        }
        fclose(fd);
    }

    wait_exit(0);
    return(0);
}




void xfire_pwd(u8 *pwd) {
    blf_ctx ctx;
    int     i,
            pwdlen;
    u8      key[] = "05c58aa143ecd48889d9fba58069ec54",
            swap[4],
            *o,
            *p;

    // simple hex2byte
    o = pwd;
    for(p = pwd; *p; p += 2) {
        if(sscanf(p, "%02x", &i) != 1) break;
        *o++ = i;
    }
    pwdlen = o - pwd;

    // endianes stuff: *(int *)pwd = ntohl(*(int *)pwd);
    for(i = 0; (i + 4) <= pwdlen; i += 4) {
        swap[0] = pwd[i + 3];
        swap[1] = pwd[i + 2];
        swap[2] = pwd[i + 1];
        swap[3] = pwd[i + 0];
        memcpy(pwd + i, swap, 4);
    }

    // blowfish decryption
    blf_key(&ctx, key, strlen(key) + 1);
    blf_dec(&ctx, (void *)pwd, pwdlen / 8);

    // endianes stuff: *(int *)pwd = htonl(*(int *)pwd);
    for(i = 0; (i + 4) <= pwdlen; i += 4) {
        swap[0] = pwd[i + 3];
        swap[1] = pwd[i + 2];
        swap[2] = pwd[i + 1];
        swap[3] = pwd[i + 0];
        memcpy(pwd + i, swap, 4);
    }
    pwd[pwdlen] = 0;
}



void std_err(void) {
    perror("\nError");
    wait_exit(1);
}



void wait_exit(int ret) {
    fprintf(stderr, "\npress RETURN to exit\n");
    fgetc(stdin);
    exit(ret);
}


